'use strict';
var traverse = require('babel-traverse').default;
var t = require('babel-types');

function getFunctionName(path) {
	return (path.node.id || {}).name;
}

module.exports = function (options) {
	var excludes = options.exclude || {};
	var instrumentationExpressionCall = t.memberExpression(
		t.memberExpression(
			t.identifier('window'),
			t.identifier('kieker')
		),
		t.identifier('instrumentFunction')
	);

	// Handle excludes
	var shouldBeSkipped = function (path) {
		var functionName = getFunctionName(path);
		if (excludes.anonymous && !functionName) {
			// This function is anonymous, so we skip the excludes
			return true;
		}

		if (excludes.name && (functionName || '').match(excludes.name)) {
			return true;
		}

		return false;
	};

	var findFunctionVisitor = {
		FunctionDeclaration: function (path) {
			if (shouldBeSkipped(path)) {
				return;
			}

			var node = path.node;

			// Move every FunctionDeclaration to FunctionExpression
			var newNode = t.functionExpression(
				node.id,
				node.params,
				node.body,
				node.generator,
				node.async
			);

			// TODO: for excludes of functionDeclarations we would need to store a list of these IDs
			path.replaceWith(t.variableDeclaration('var', [
				t.variableDeclarator(
					path.node.id,
					newNode
				)
			]));

			// traverse(newNode, findFunctionVisitor, path.scope, path);
		},
		FunctionExpression: function (path) {
			if (shouldBeSkipped(path)) {
				return;
			}

			// Traversing further leads to infinite loop as same node is found again
			path.skip();
			var node = path.node;

			// Move every FunctionDeclaration to FunctionExpression
			var newNode = t.functionExpression(
				node.id,
				node.params,
				node.body,
				node.generator,
				node.async
			);

			path.replaceWith(
				t.callExpression(instrumentationExpressionCall, [newNode])
			);

			traverse(newNode, findFunctionVisitor, path.scope, path);
		}
	};

	return function (ast) {
		traverse(ast, findFunctionVisitor);
		return ast;
	};
};
